/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
BookmarkTools.include("external/luceneQueryValidator.js");

BookmarkTools.namespace("indexing");

BookmarkTools.indexing.AUTOMATIC = 0;
BookmarkTools.indexing.MANUAL = 1;

BookmarkTools.indexing.DELAY_ONE_WEEK = 604800000;
BookmarkTools.indexing.DELAY_TWO_WEEK = 1209600000;
BookmarkTools.indexing.DELAY_THREE_WEEK = 1814400000;
BookmarkTools.indexing.DELAY_FOUR_WEEK = 2419200000;

BookmarkTools.indexing.STATUS_SUCCESS = 0;
BookmarkTools.indexing.STATUS_CORRUPTED = 1;
BookmarkTools.indexing.STATUS_UNTRACEABLE = 2;

BookmarkTools.indexing.TYPE_URL = 0;
BookmarkTools.indexing.TYPE_CONTENT = 1;

/**
	@class Basic indexer that wraps the clucene component into the js world. <br />
*/
BookmarkTools.indexing.Index = {

	path : "",
	delay : BookmarkTools.indexing.DELAY_ONE_WEEK, //default delay one week (in millis)
	mode : BookmarkTools.indexing.MANUAL, //default mode
	filter : null,
	crawler : null,
	
	indexWriter : null,
	indexReader : null,

	//TODO: implement _init()
	init : function(){
		var instWriter = Cc["@bookmarktools.mozdev.org/search/indexWriter;1"].createInstance();
		this.indexWriter = instWriter.QueryInterface(Ci.ICLWriter);
			
		var instReader = Cc["@bookmarktools.mozdev.org/search/indexReader;1"].createInstance();
		this.indexReader = instReader.QueryInterface(Ci.ICLReader);
	
		this.indexReader.setPath(BookmarkTools.utils.FileIO.getSearchDir().path); 
		this.setMode(BookmarkTools.utils.Prefs.getInt("buildType")); 
		
		switch(BookmarkTools.utils.Prefs.getInt("indexDelay")){
			case 1: this.delay = BookmarkTools.indexing.DELAY_ONE_WEEK; break;
			case 2: this.delay = BookmarkTools.indexing.DELAY_TWO_WEEK; break;
			case 3: this.delay = BookmarkTools.indexing.DELAY_THREE_WEEK; break;
			case 4: this.delay = BookmarkTools.indexing.DELAY_FOUR_WEEK; break;
			default: throw 'Error: Unknown delay!';
		}
	},
	
	/**
		Sets the current build mode.
		@param mode (automatic or manually)
	 */
	setMode : function(mode){
		this.mode = mode;
		switch(this.mode){
			case BookmarkTools.indexing.AUTOMATIC: 
				this.path = BookmarkTools.utils.FileIO.getBuildDir().path;
				break;
			case BookmarkTools.indexing.MANUAL:
				this.path = BookmarkTools.utils.FileIO.getSearchDir().path;
				break;
			default: throw 'Error: Unknown build type!';
		}
	
		this.indexWriter.setPath(this.path);
	},

	/**
		Sets the content filter.
		@param  {BookmarkTools.indexing.Filter} filter
	 */
	setFilter : function(filter){
		this.filter = filter;
		this.filter.onComplete = this.onComplete;
		this.filter.onDone = this.onDone;
		this.filter.onAbort = this.onAbort;
	},

	/**
		Sets crawler reference.
		@param {BookmarkTools.indexing.Crawler} crawler
	 */
	setCrawler : function(crawler){
		this.crawler = crawler;
		this.crawler.onComplete = this.onComplete;
		this.crawler.onDone = this.onDone;
		this.crawler.onAbort = this.onAbort;
	},
	
	/**
		Function pointer to handle the crawler/filter onComplete event.
		@param {string} combinedContent
		@param bookmark {<a href="https://developer.mozilla.org/en/nsINavHistoryResultNode">nsINavHistoryResultNode</a>}
		@param {string} language
	 */
	onComplete : function(combinedContent, bookmark, language){
		//don't index livemarks
		if(!BookmarkTools.common.Book.isLivemark(bookmark)){
			BookmarkTools.indexing.Index.indexWriter.appendDocument(
				BookmarkTools.common.Book.getURI(bookmark),
				BookmarkTools.common.Book.getName(bookmark),
				combinedContent,
				language
			);
			if(BookmarkTools.indexing.Index.mode == BookmarkTools.indexing.AUTOMATIC){
				var file = BookmarkTools.utils.Prefs.getFile('alreadyAddedBookmarks');
				BookmarkTools.utils.FileIO.writeLineToFile(file, BookmarkTools.common.Book.getURI(bookmark) + BookmarkTools.utils.StringUtils.LF);
			}
		}
	},

	/**
		Function pointer to handle the crawler/filter onDone event.
	 */	
	onDone : function(){	
		if(BookmarkTools.indexing.Index.filter){
			BookmarkTools.indexing.Index.filter.flushDuplicates();
			if(BookmarkTools.indexing.Index.filter.view){
				BookmarkTools.indexing.Index.filter.view._init();
				if(BookmarkTools.indexing.Index.mode == BookmarkTools.indexing.MANUAL) BookmarkTools.panel.index.startLookup();
			}
			BookmarkTools.indexing.Index.filter._init();
		}
		BookmarkTools.indexing.Index.indexWriter.optimize();
		
		if(BookmarkTools.indexing.Index.mode == BookmarkTools.indexing.AUTOMATIC){
			BookmarkTools.utils.Prefs.setBool('wasUpdated', true);
			BookmarkTools.utils.Prefs.setBool('firstRun', true);
			BookmarkTools.utils.Prefs.remove("alreadyAddedBookmarks");
		}
			
		var currentTime = new Date().getTime();
		BookmarkTools.utils.Prefs.setChar('intIndexDate', currentTime);
	},

	/**
		Function pointer to handle the crawler/filter onAbort event.
	 */
	onAbort : function(){
		if(BookmarkTools.indexing.Index.filter){
			BookmarkTools.indexing.Index.filter.flushDuplicates();
			if(BookmarkTools.indexing.Index.filter.view){
				BookmarkTools.indexing.Index.filter.view._init();
				if(BookmarkTools.indexing.Index.mode == BookmarkTools.indexing.MANUAL) 
					BookmarkTools.panel.index.startLookup();
			}
			BookmarkTools.indexing.Index.filter._init();
		}
		BookmarkTools.indexing.Index.indexWriter.optimize();
	},

	/**
		Evaluate if the current time (millis) plus delay is valid for rebuild index data.
	 */	
	isValidBuildTime : function(){
		if(this.mode == BookmarkTools.indexing.AUTOMATIC){
			var lastBuildTime = 0;
			try {
				lastBuildTime = BookmarkTools.utils.NumberUtils.parseFloat(BookmarkTools.utils.Prefs.getChar('intIndexDate'));
			} catch(e){}
			
			var currentTime = new Date().getTime();
			
			if(isNaN(lastBuildTime)){
				lastBuildTime = 0
			}
		
			if((lastBuildTime + this.delay) <= currentTime){
				return true;
			}
		}
		return false;
	},

	/**
		Acquires an index lock for one browser instance.
	 */	
	acquire : function(){
		var key = new Date().getTime();
		var browserWindows = BookmarkTools.utils.NativeComponents.nsIWindowMediator("navigator:browser");
		
		return browserWindows.length == 1 ? key : -1;
	},
		
	/**
		Start building index data.
	 */	
	_build : function(){
		if(this.filter){
			this.filter.go();
		}else {
			if(this.crawler){ //TODO: check -> useful case?
				this.crawler._build();
			}else {
				throw 'Error: No filter available!';
			}
		}
	},

	/**
		Executes index search.
		@param pattern {string}
		@param container {object}
		@param chunk {number}
		@param chunkSize {number}
	 */	
	_search : function(pattern, container, chunk, chunkSize){
		var max = 0;
		if(this.indexReader.exists()){ //index exists ?
			// validate pattern
			setAlertUser(true);
		
			var query = !doCheckLuceneQueryValue(pattern) ? BookmarkTools.utils.StringUtils.EMPTY : pattern;
			query = BookmarkTools.utils.StringUtils.trim(query);

			if(query.length > 2){

				var array = BookmarkTools.utils.NativeComponents.nsIMutableArray();
				max = this.indexReader.search(query, array, chunk, chunkSize);

				//clear result tree
				container.clear();

				//draw results
				var e = array.enumerate();
				var resultNode, nodes, node;
				
				while (e.hasMoreElements()) {
				
					resultNode = e.getNext().QueryInterface(Ci.IResultNode);
					nodes = BookmarkTools.common.Book.getNode(resultNode.primaryKey);
					
					/** if 'nodes' is empty the url isn't bookmarked */
					if(BookmarkTools.utils.ArrayUtils.isNotEmpty(nodes)){
					
						node = nodes[0];
						container.add(
							BookmarkTools.common.Book.getName(node), 
							BookmarkTools.common.Book.getURI(node),
							BookmarkTools.common.Book.getIcon(node), 
							resultNode.language,
							resultNode.score == 0 ? BookmarkTools.utils.StringUtils.EMPTY : resultNode.score
						);
						
					}else {
						//TODO: string bundle -> '<Untraceable Bookmark>'
						container.add(
							'<Untraceable Bookmark>', 
							resultNode.primaryKey,
							"chrome://bookmarktools/skin/exclamation.png", 
							BookmarkTools.utils.StringUtils.EMPTY,
							resultNode.score == 0 ? BookmarkTools.utils.StringUtils.EMPTY : resultNode.score
						);
					}
				}
				
			}else {
				container.clear();
			}
		}else {
			alert('Index is empty!');
		}
		return max;
	},
	
	// returns the current index size, that means the number of indexed documents
	_size : function(){
		return this.indexReader.size();
	},
	
	// deletes all or a subset of index documents
	//
	//	@param {array} nodes index array
	//
	_delete : function(nodes){

		try {

			if(nodes.length == this._size() || BookmarkTools.utils.ArrayUtils.isEmpty(nodes)){
				
				//it is possible to delete every document but
				//faster to clear the whole directory
				BookmarkTools.utils.FileIO.clear(BookmarkTools.utils.FileIO.getSearchDir());
				
			}else { //subset

				if(!BookmarkTools.utils.Prefs.getBool('isRunning')){
					//cleanup
					var i, value;
					for(i = 0, l = nodes.length ;i < l; i++){
						//delete index document
						value = nodes[i];
						BookmarkTools.indexing.Index.indexWriter.deleteDocument(value); 
					}

					this.indexWriter.optimize();
				}

			}
			
		}catch(e) {
			BookmarkTools.reportError(e);
		}
		
	},

	// return the index as mutable array
	_lookup : function(){
		var array = BookmarkTools.utils.NativeComponents.nsIMutableArray();
		this.indexReader.lookup(array);
		return array;
	},
	
	// if build is running this function will abort the crawler
	_abort : function(){
		if(!this.filter) throw 'Error: No filter available!';
		if(!this.filter.crawler) throw 'Error: No crawler available!';
		
		this.filter.crawler._abort();
		// if indexer was running reset this flag
		if(BookmarkTools.utils.Prefs.getBool('isRunning'))
			BookmarkTools.utils.Prefs.setBool('isRunning', false);
	}

};

BookmarkTools.indexing.Index.init(); 

BookmarkTools.indexing.Filter = {

	uniqueContent : null,
	duplicateSelector : null,
	
	countDuplicate : 0,
	countError : 0,
	count : 0,
	max : 0,
	
	crawler : null,
	view : null,
	
	check : false,
	
	logger : null,
	
	init : function(){
		BookmarkTools.indexing.Parser.init();
		this.uniqueContent = [];

		this.duplicateSelector = new BookmarkTools.dataTypes.ExtendedHash();
		this.logger = new BookmarkTools.Logger({
			object: this
		});
	},
	
	_init : function(){
		BookmarkTools.indexing.Parser._init();
		this.uniqueContent = null;

		this.duplicateSelector = null;
		this.countDuplicate = 0;
		this.countError = 0;
		this.count = 0;
		this.max = 0;
		this.check = false;
	},
	
	setCrawler : function(crawler, count, max){
		this.crawler = crawler;
		this.crawler.onComplete = this.filter;
		this.crawler.onError = this.updateError;
		this.crawler.onDone = this.onDone;
		this.crawler.onAbort = this.onAbort;
		
		if(arguments.length == 3){
			this.count = count;
			this.max = max;
		}else {
			this.max = this.crawler.bookmarks.length();
		}
	},
	
	getBookmarksFromFile : function(){
		var file = BookmarkTools.utils.Prefs.getFile('alreadyAddedBookmarks');
		var alreadyAddedBookmarks = BookmarkTools.utils.FileIO.readFileLineByLine(file);
	
		var bookmarks = BookmarkTools.common.Book.getResourceHash();
		var notAddedBookmarks = new BookmarkTools.dataTypes.ExtendedHash();
		
		if(alreadyAddedBookmarks.isEmpty()){
			return {
				currentIndexed : 0,
				allBookmarks : bookmarks.length(),
				remainingBookmarks : bookmarks
			};
		}

		bookmarks.each(function(key, value){
			if(!alreadyAddedBookmarks.contains(key)){
				notAddedBookmarks.put(key, value);
			}
		});
		
		return {
			currentIndexed : alreadyAddedBookmarks.length(),
			allBookmarks : bookmarks.length(),
			remainingBookmarks : notAddedBookmarks
		};
	},
	
	flushDuplicates : function(){
		BookmarkTools.indexing.Filter.duplicateSelector.each(function(key, value){
			BookmarkTools.indexing.Filter.updateDuplicate(key, value);
		});
	},
	
	onComplete : function(combinedContent, bookmark, language){},
	
	onDone : function(){
		BookmarkTools.indexing.Filter.flushDuplicates();
		if(BookmarkTools.indexing.Filter.view)
			BookmarkTools.indexing.Filter.view._init();
		BookmarkTools.indexing.Filter._init();
	},
	
	onAbort : function(){
		BookmarkTools.indexing.Filter.flushDuplicates();
		if(BookmarkTools.indexing.Filter.view)
			BookmarkTools.indexing.Filter.view._init();
		BookmarkTools.indexing.Filter._init();
	},
	
	setView : function(view){
		this.view = view;
	},
	
	setCheck : function(check){
		this.check = check;
	},
	
	go : function(){
		if(!this.crawler) throw 'Error: No crawler available!';

		//url duplicates loaded one time at startup in automatic build mode
		if((BookmarkTools.indexing.Index.mode == BookmarkTools.indexing.AUTOMATIC && BookmarkTools.utils.Prefs.getBool('firstRun')) ||
			BookmarkTools.indexing.Index.mode == BookmarkTools.indexing.MANUAL){
			
			var duplicateURLs = this.crawler.bookmarks.trash();
			var i, node, uri;
			
			for(i = 0, l = duplicateURLs.length ;i < l; i++){
				
				node = duplicateURLs[i];
				uri = BookmarkTools.common.Book.getURI(node);
				
				if(BookmarkTools.indexing.Filter.duplicateSelector.contains(uri)){
					BookmarkTools.indexing.Filter.duplicateSelector.get(uri).push({bookmark: node, type: BookmarkTools.indexing.TYPE_URL});
				}else {
					BookmarkTools.indexing.Filter.duplicateSelector.put(uri, [{bookmark: node, type: BookmarkTools.indexing.TYPE_URL}]);
				}
			}
		}
		
		//update counter because automatic indexing
		//is done in 0-n steps and every step produces duplicates
		if(BookmarkTools.indexing.Index.mode == BookmarkTools.indexing.AUTOMATIC){
			this.countDuplicate = BookmarkTools.utils.Prefs.getBranch('dupBm').length;
			this.countError = BookmarkTools.utils.Prefs.getBranch('invBm').length;
			
			// BrowserView does not need any uri, so setProgress(..) has only 2 arguments
			//TODO:  BookmarkTools.indexing.BrowserView is not defined at this point 
			if(BookmarkTools.indexing.Filter.view && 
				BookmarkTools.indexing.Filter.view == BookmarkTools.indexing.BrowserView) 
				BookmarkTools.indexing.Filter.view.setProgress(
					BookmarkTools.indexing.Filter.count, 
					BookmarkTools.indexing.Filter.max
				);
			else throw 'Error: Invalid view type for automatic mode!'
		}
		this.crawler._build();
	},
	
	//overwrites crawlers onNext and onUpdate
	filter : function(responseText, mimeType, charSet, bookmark){
		try{
			if(!BookmarkTools.indexing.Filter.check){
				//returns stopword filtered word list and language
				var parsedDocument = BookmarkTools.indexing.Parser.parse(
					mimeType, charSet, responseText
				);

				// index content which is build up in the following steps
				var indexableContent = BookmarkTools.utils.StringUtils.EMPTY;
				
				//step 1: word list is empty
				if(BookmarkTools.utils.ArrayUtils.isEmpty(parsedDocument.words)){
					indexableContent = BookmarkTools.indexing.Filter.combineWithoutContent(bookmark);
				}else {
				
					var trimmedContent = BookmarkTools.utils.StringUtils.trim(
						parsedDocument.words.join(BookmarkTools.utils.StringUtils.SPACE)
					);
				
					//step 2: test if word list containing whitespaces
					if(BookmarkTools.utils.StringUtils.isEmpty(trimmedContent)){
						indexableContent = BookmarkTools.indexing.Filter.combineWithoutContent(bookmark);
					}else {
					
						//step 3: check if content has duplicates
						if(!BookmarkTools.indexing.Filter.isDuplicate(trimmedContent, bookmark)){
							BookmarkTools.indexing.Filter.onComplete(
								BookmarkTools.indexing.Filter.combineContent(trimmedContent, bookmark), 
								bookmark,
								parsedDocument.languages.toString()
							);
						}
					}
				}
				
				//step 4: checks if resource had content, otherwise just index bookmarks
				//properties
				if(BookmarkTools.utils.StringUtils.isNotEmpty(indexableContent)){
					BookmarkTools.indexing.Filter.onComplete(
						indexableContent, 
						bookmark,
						parsedDocument.languages.toString()
					);
				}
			}
			
			var uri = BookmarkTools.common.Book.getURI(bookmark);
			BookmarkTools.indexing.Filter.count++;
			
			if(BookmarkTools.indexing.Filter.view)
				BookmarkTools.indexing.Filter.view.setProgress(
					BookmarkTools.indexing.Filter.count, 
					BookmarkTools.indexing.Filter.max, 
					uri
				);
				
			BookmarkTools.indexing.Filter.logger.info(uri, arguments.callee);
		}catch(e){
			BookmarkTools.reportError(e);
		}
	},
	
	combineContent : function(content, bookmark){
		content += BookmarkTools.utils.StringUtils.SPACE;
		content += BookmarkTools.common.Book.getName(bookmark);
		content += BookmarkTools.utils.StringUtils.SPACE;
		content += BookmarkTools.common.Book.getURI(bookmark);
		content += BookmarkTools.utils.StringUtils.SPACE;
		content += BookmarkTools.common.Book.getDesc(bookmark);
		content += BookmarkTools.utils.StringUtils.SPACE;
		content += BookmarkTools.common.Book.getKeys(bookmark);
		return BookmarkTools.utils.StringUtils.trim(content);
	},
	
	combineWithoutContent : function(bookmark){
		var combined = BookmarkTools.common.Book.getName(bookmark);
		combined += BookmarkTools.utils.StringUtils.SPACE;
		combined += BookmarkTools.common.Book.getURI(bookmark);
		combined += BookmarkTools.utils.StringUtils.SPACE;
		combined += BookmarkTools.common.Book.getDesc(bookmark);
		combined += BookmarkTools.utils.StringUtils.SPACE;
		combined += BookmarkTools.common.Book.getKeys(bookmark);
		return BookmarkTools.utils.StringUtils.trim(combined);
	},

	isDuplicate : function(trimmedContent, bookmark){
	
		if(BookmarkTools.indexing.Filter.uniqueContent[trimmedContent] == undefined){
			BookmarkTools.indexing.Filter.uniqueContent[trimmedContent] = BookmarkTools.common.Book.getURI(bookmark);
			return false;
		}
		
		var original = BookmarkTools.indexing.Filter.uniqueContent[trimmedContent];
		if(this.duplicateSelector.contains(original)){
			this.duplicateSelector.get(original).push({bookmark: bookmark, type: BookmarkTools.indexing.TYPE_CONTENT});
		}else {
			this.duplicateSelector.put(original, [{bookmark: bookmark, type: BookmarkTools.indexing.TYPE_CONTENT}]);
		}
	
		return true;
	},

	updateDuplicate : function(uri, equals){
		var i;
		
		if(BookmarkTools.utils.Prefs.getBool('cleanupDuplicates')){
		
			for(i = 0, l = equals.length ;i < l; i++){
				BookmarkTools.common.Book.removeBookmark(equals[i].bookmark);
			}
	
		}else {
			var json;
			var duplicate_tree = BookmarkTools.$('bookmarkTools.pane.duplicateTree');
			var itemCount = BookmarkTools.$('bookmarkTools.pane.duplicateCtr');
			
			for(i = 0, l = equals.length ;i < l; i++){
				json = BookmarkTools.utils.JSONUtils.toJSON(
					{
						uri: BookmarkTools.common.Book.getURI(equals[i].bookmark),
						type: equals[i].type
					}
				);
				BookmarkTools.utils.Prefs.setChar('dupBm' + BookmarkTools.indexing.Filter.countDuplicate++, json);
				
				if(duplicate_tree){
					duplicate_tree.add(
						BookmarkTools.common.Book.getName(equals[i].bookmark),
						BookmarkTools.common.Book.getURI(equals[i].bookmark),
						BookmarkTools.common.Book.getIcon(equals[i].bookmark),
						'dupBm' + (BookmarkTools.indexing.Filter.countDuplicate - 1)
					);
					itemCount.value = '(' + duplicate_tree.size + ')';
				}
			}
		}
	},
	
	updateError : function(status, statusText, bookmark){
		BookmarkTools.indexing.Filter.count++;
		var json = BookmarkTools.utils.JSONUtils.toJSON(
			{
				uri: BookmarkTools.common.Book.getURI(bookmark),
				status: status,
				statusText: statusText
			}
		);
		
		BookmarkTools.indexing.Filter.logger.info(BookmarkTools.common.Book.getURI(bookmark), arguments.callee);
		BookmarkTools.utils.Prefs.setChar('invBm' + BookmarkTools.indexing.Filter.countError++, json);
	
		var error_tree = BookmarkTools.$('bookmarkTools.pane.errorTree');
		var other_tree = BookmarkTools.$('bookmarkTools.pane.otherTree');
		
		if((error_tree || other_tree) && typeof BookmarkTools.panel.errors.updateView != 'undefined')
			BookmarkTools.panel.errors.updateView(status, statusText, bookmark, BookmarkTools.utils.Prefs.getInt('errorView'));
		
		//if error type equals 'OTHER' the bookmark is valid but no content available
		//-> in this case only the bookmark properties combined for index document
		if(status <= -400){
			if(!BookmarkTools.indexing.Filter.check){
				var combined = BookmarkTools.indexing.Filter.combineWithoutContent(bookmark);
				BookmarkTools.indexing.Filter.onComplete(
					combined, 
					bookmark,
					BookmarkTools.utils.StringUtils.EMPTY /* unable to receive language here */
				);
			}	
		}
	}

};

BookmarkTools.indexing.View = {

	doc : null,
	win : null,
	panel : null,
	
	init : function(){
		var w = BookmarkTools.utils.NativeComponents.nsIWindowMediator('bookmarkTools-manual-indexing');
		this.win = w[0];
		this.doc = w[0].document;
		this.panel = {
			label: BookmarkTools.indexing.View.doc.getElementById('bookmarkTools.progress.info'),
			progressLabel: BookmarkTools.indexing.View.doc.getElementById('bookmarkTools.progress.label'),
			progressBar: BookmarkTools.indexing.View.doc.getElementById('bookmarkTools.progress.bar')
		};
	},
	
	_init : function(){
		this.win.close();
		this.win = null;
		this.doc = null;
		this.panel = null;
	},

	percent : function(i, j){
		return Math.floor((i / j) * 100);
	},
	
	formatURI : function(i, j, uri, maxLength){
		var range = '[' + i + '/' + j + '] ';
		return range + BookmarkTools.utils.StringUtils.truncate(uri, maxLength, ' [..]');
	},
	
	formatProgress : function(percent){
		return percent + '%';
	},
	
	setProgress : function(i, j, uri){
		this.panel.label.value = this.formatURI(i, j, uri, 30);
		var val = this.percent(i, j);
		this.panel.progressLabel.value = this.formatProgress(val);
		this.panel.progressBar.value = val;
	}

};

BookmarkTools.indexing.BrowserView = {
	
	doc : null,
	win : null,
	panel : null,
	
	init : function(){
		this.win = BookmarkTools.utils.NativeComponents.nsIBrowserWindow();
		this.doc = this.win.document;
		this.panel = {
			progressLabel: BookmarkTools.indexing.BrowserView.doc.getElementById('bookmarkTools.autoprogress.label'),
			progressBar: BookmarkTools.indexing.BrowserView.doc.getElementById('bookmarkTools.autoprogress.bar')
		};
		BookmarkTools.$('bookmarkTools.autoProgressOverlay').hidden = false;
	},
	
	_init : function(){
		BookmarkTools.$('bookmarkTools.autoProgressOverlay').hidden = true;
	},
	
	percent : function(i, j){
		return Math.floor((i / j) * 100);
	},
	
	formatProgress : function(percent){
		return percent + '%';
	},
	
	setProgress : function(i, j){
		var val = this.percent(i, j);
		this.panel.progressLabel.value = this.formatProgress(val);
		this.panel.progressBar.value = val;
	}
};
