/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("common.Book");

Cu.import("resource://gre/modules/utils.js");

/**
	@requires bookmarkToolsGlobals.js, bookmarkToolsJSHelper.js
	@class
	This class was splited into two implementations that handles bookmarks via places or rdf service. Seamonkey and 
	older versions of firefox 2.x can use the rdf instance and firefox 3.0 or higher can use places. <br />
	
	@example
	Receiving an array of bookmarks:
		var bookmarks = BookmarkTools.common.Book.getResourceArray();
		for(var i = 0, l = bookmarks.length ;i < l; i++){
			var bookmark = bookmarks[i];
			alert(BookmarkTools.common.Book.getName(bookmark));
			alert(BookmarkTools.common.Book.getURI(bookmark));
		}
*/
BookmarkTools.common.Book = {

	historyService : Cc["@mozilla.org/browser/nav-history-service;1"].getService(Ci.nsINavHistoryService),
	bookmarkService : Cc["@mozilla.org/browser/nav-bookmarks-service;1"].getService(Ci.nsINavBookmarksService),
	livemarkService : Cc["@mozilla.org/browser/livemark-service;2"].getService(Ci.nsILivemarkService),
	faviconService : Cc["@mozilla.org/browser/favicon-service;1"].getService(Ci.nsIFaviconService),
	annotationService : Cc["@mozilla.org/browser/annotation-service;1"].getService(Ci.nsIAnnotationService),

	RESULT_TYPE_URI			: 0,
	RESULT_TYPE_FOLDER		: 6,
	RESULT_TYPE_SEPARATOR	: 7,

	/** Common type attribute: bookmark */
	TYPE_BOOKMARK		: 0,
	/** Common type attribute: folder */
	TYPE_FOLDER			: 1,
	/** Common type attribute: livemark */
	TYPE_LIVEMARK		: 2,
	/** Common type attribute: separator */
	TYPE_SEPARATOR		: 3,
	/** Common type attribute: unknown */
	TYPE_UNKNOWN 		: 4,

	//+------------------------------------------------------------------------------------
	// Base types
	//+------------------------------------------------------------------------------------

	getBookmarkOptions : function(){
		var options = this.historyService.getNewQueryOptions();
		options.queryType = options.QUERY_TYPE_BOOKMARKS; 
		options.expandQueries = true;
		options.includeHidden = true;
		return options;
	},

	getBookmarkQuery : function(folders){
		var query = this.historyService.getNewQuery();
		query.setFolders(folders, folders.length);
		
		return query;
	},

	/**
		Returns the number of all bookmarks.
		@return {integer} size number of bookmarks
	*/
	getSize : function(){
		return this.getResourceArray().length;
	},
	
	/**
		Returns a list of nodes that matches against the given uri.
		@param {string} uri bookmarks uri property
		@ see <a href="https://developer.mozilla.org/en/nsINavHistoryResultNode">nsINavHistoryResultNode</a>
		@return {array} list of nodes
	*/
	getNode : function(uri){
		var query = this.getBookmarkQuery([this.bookmarkService.placesRoot]);
		query.uri = BookmarkTools.utils.StringUtils.nsiURI(uri);
		var options = this.getBookmarkOptions();
		
		var result = this.historyService.executeQuery(query, options);
		var node = result.root.QueryInterface(Ci.nsINavHistoryContainerResultNode);
		
		return this.toBaseType([], result);

	},
	
	/**
		Returns the parent of a node that matched against the given uri. If
		the uri isn't unique the first match is returned.
		@param {string} uri bookmarks uri property
		@ see <a href="https://developer.mozilla.org/en/nsINavHistoryResultNode">nsINavHistoryResultNode</a>
		@return {object} parent node
	*/
	getParent : function(uri){
		var ids = this.bookmarkService.getBookmarkIdsForURI(BookmarkTools.utils.StringUtils.nsiURI(uri), {});
		return this.bookmarkService.getFolderIdForItem(ids.pop());
	},
	
	/**
		Returns all bookmarks as array.
		@return {array} list of nodes
	*/
	getResourceArray : function(){
		return this.fill([]);
	},
	
	/**
		Returns bookmarks as hash map (key = uri). Duplicates are stored in
		the trash property of this map.
		@return {object} map of nodes
	*/
	getResourceHash : function(){ 
		return this.fill(new BookmarkTools.dataTypes.ExtendedHash());
	},
	
	/**
		Returns bookmarks as hash map (key = id). Duplicates are stored in
		the trash property of this map.
		@return {object} map of nodes
	*/
	getResourceIDHash : function(){
		return this.fill(new BookmarkTools.dataTypes.ExtendedHash(), 'ID');
	},

	/**
		Binds a function which is called for every bookmark.
		@param {function} callback function pointer
	*/
	getResourceCallback : function(callback){
		if(!callback) throw 'Error: No callback function added!';
		this.fill(callback);
	},

	fill : function(object, id){
		var options = this.getBookmarkOptions();
		var query = this.getBookmarkQuery([this.bookmarkService.placesRoot]);

		var result = this.historyService.executeQuery(query, options);
		return this.toBaseType(object, result, id);
	},
	
	toBaseType : function(object, result, id){
		var root = result.root.QueryInterface(Ci.nsINavHistoryContainerResultNode);
		
		root.containerOpen = true; 
		var i;
		for (i = 0, l = root.childCount; i < l; i++) {		
			this.collectNodes(root.getChild(i), object, id);
		}
		root.containerOpen = false;
		
  		return object;
	},

	collectNodes : function(node, object, id){
		switch(node.type){
			case this.RESULT_TYPE_URI : {
				this.put(node, object, id);
				break;
			}
			case this.RESULT_TYPE_FOLDER : {
				if(this.isLivemark(node)){
					this.put(node, object, id);
				}else {
					node.QueryInterface(Ci.nsINavHistoryContainerResultNode);

					node.containerOpen = true;
					try{
						for (var i = 0, l = node.childCount; i < l; i++) {	
							this.collectNodes(node.getChild(i), object, id);
						}
					}catch(e){}
					node.containerOpen = false;
				}
				break;
			}
			default : {
				break;
			}
		}
	},

	put : function(node, object, id){
		if(BookmarkTools.utils.ObjectUtils.isArray(object)){
			object.push(node);
		}else {
			if(object instanceof BookmarkTools.dataTypes.ExtendedHash){
				if(id){
					object.put(node.itemId, node);
				}else {
					object.put(this.getURI(node), node);
				}
			}else {
				if(BookmarkTools.utils.ObjectUtils.isFunction(object)){
					object(node);
				}else {
					throw 'Unknown base type : (' + (typeof object) + ')';
				}
			}
		}
	},

	resolveURL : function(nsiURI){
		var host = nsiURI.host;
		var path = nsiURI.path;
		var protocol = "http://";
		return protocol + host + "/" + path;
	},

	//+------------------------------------------------------------------------------------
	// Add
	//+------------------------------------------------------------------------------------

	/**
		Adds a bookmark with name and uri property.
		@param {string} name bookmarks name
		@param {string} uri bookmarks uri
	*/
	addBookmark : function(name, uri){
		var folderID = this.bookmarkService.bookmarksMenuFolder;
		this.bookmarkService.insertBookmark(folderID, BookmarkTools.utils.StringUtils.nsiURI(uri), -1, name);
	},

	//TODO
	addFolder : function(){

	},

	//TODO
	addLivemark : function(){

	},

	//TODO
	addSeparator : function(){

	},

	//+------------------------------------------------------------------------------------
	// Remove
	//+------------------------------------------------------------------------------------

	/**
		Removes given bookmark.
		@param {object} node bookmark node
		@ see <a href="https://developer.mozilla.org/en/nsINavHistoryResultNode">nsINavHistoryResultNode</a>
	*/
	removeBookmark : function(node){
		this.bookmarkService.removeItem(node.itemId);
	},

	//TODO: test
	removeFolder : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//TODO: test
	removeLivemark : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//TODO: test
	removeSeparator : function(aItemId){
		this.bookmarkService.removeItem(aItemId);
	},

	//+------------------------------------------------------------------------------------
	// Properties
	//+------------------------------------------------------------------------------------

	/**
		Returns the type of a given bookmark node.
		@param {object} node bookmark node
		@return {integer} bookmarks type property
	*/
	getType : function(node){
		switch(node.type){
			case this.RESULT_TYPE_URI : {
				return this.TYPE_BOOKMARK;
			}
			case this.RESULT_TYPE_FOLDER : {
				if(PlacesUtils.nodeIsLivemarkContainer(node)){
					return this.TYPE_LIVEMARK;
				}
				return this.TYPE_FOLDER;
			}
			case this.RESULT_TYPE_SEPARATOR : {
				return this.TYPE_SEPARATOR;
			}
			default : {
				return this.TYPE_UNKNOWN;
			}
		}
	},
	
	/**
		Returns the bookmarks name.
		@param {object} node bookmark instance
		@return bookmarks name property
	*/
	getName : function(node){
		return node ? (node.title ? node.title : BookmarkTools.utils.StringUtils.EMPTY) : BookmarkTools.utils.StringUtils.EMPTY;
	},

	/**
		Returns the bookmarks url.
		@param {object} node bookmark instance
		@return bookmarks url property
	*/
	getURL : function(node){
		return node ? (node.uri ? node.uri : BookmarkTools.utils.StringUtils.EMPTY) : BookmarkTools.utils.StringUtils.EMPTY;
	},

	/**
		Returns the livemarks furl (feed url).
		@param {object} node bookmark instance
		@return livemarks furl property
	*/
	getFURL : function(node){
		if(node){
			var url = this.livemarkService.getFeedURI(node.itemId);
			var furl = url ? this.resolveURL(url) : null;
			return furl ? furl : BookmarkTools.utils.StringUtils.EMPTY;
		}
		return BookmarkTools.utils.StringUtils.EMPTY;
	},

	/**
		Returns in case of livemarks the furl property and otherwise the bookmarks url.
		@param {object} node bookmark instance
		@return livemarks furl or bookmarks url property
	*/
	getURI : function(node){
		if(node){
			if(this.isLivemark(node)){
				return this.getFURL(node);
			}
			return this.getURL(node);
		}
		return BookmarkTools.utils.StringUtils.EMPTY;
	},

	/**
		Returns the bookmarks keys.
		@param {object} node bookmark instance
		@return bookmarks key property
	*/
	getKeys : function(node){
		if(node){
			var keys = this.bookmarkService.getKeywordForBookmark(node.itemId);
			return keys ? keys : BookmarkTools.utils.StringUtils.EMPTY;
		}
		return BookmarkTools.utils.StringUtils.EMPTY;
	},

	getAddDate : function(node){
		return node ? (node.dateAdded ? node.dateAdded : -1) : -1;
	},

	getLMDate : function(node){
		return node ? (node.lastModified ? node.lastModified : -1) : -1;
	},

	//TODO
	getLVDate : function(node){
		return BookmarkTools.utils.StringUtils.EMPTY;
	},

	/**
		Returns the bookmarks description.
		@param {object} node bookmark instance
		@return bookmarks description property
	*/
	getDesc : function(node){
		if(node){
			if (this.annotationService.itemHasAnnotation(node.itemId, "bookmarkProperties/description")){
				return this.annotationService.getItemAnnotation(node.itemId, "bookmarkProperties/description");
			}
		}
		return BookmarkTools.utils.StringUtils.EMPTY;
	},

	/**
		Returns the bookmarks icon.
		@param {object} node bookmark instance
		@return bookmarks icon property
	*/
	getIcon : function(node){
		var uri = BookmarkTools.utils.StringUtils.nsiURI(this.getURI(node));
		var iconUri = null;
		
		try {
			iconUri = this.faviconService.getFaviconForPage(uri);
		} catch(e) {}
		
		if(iconUri){
			var mimeType = {};
			var iconData = null;
			try {
				iconData = this.faviconService.getFaviconData(iconUri, mimeType, {});
			} catch(e) {}
			if (iconData && iconData.length > 0)
				return "data:" + mimeType.value + ";base64," + BookmarkTools.utils.StringUtils.encodeBase64(iconData);
		}
		return "chrome://browser/skin/Bookmarks-folder.png";
	},
	
	/**
		Returns true if the checked node is an livemark, otherwise false.
		@param {object} node bookmark instance
		@return true or false
	*/
	isLivemark : function(node){
		node.QueryInterface(Ci.nsINavHistoryResultNode);
		return PlacesUtils.nodeIsLivemarkContainer(node);
	},
	
	/**
		Returns true if the given url is bookmarked, otherwise false.
		@param {string} url url to check
		@return true for bookmarked urls, otherwise false
	*/
	isBookmarked : function(url){
		return PlacesUtils.bookmarks.isBookmarked(BookmarkTools.utils.StringUtils.nsiURI(url));
	}

};
